#include "testApp.h"


//--------------------------------------------------------------
void testApp::setup(){	 

	// load in sounds:
	sufjan.loadSound("sounds/iwalked.mp3");

	// the fft needs to be smoothed out, so we create an array of floats
	// for that purpose:
	fftSmoothed = new float[8192];
	for (int i = 0; i < 8192; i++){
		fftSmoothed[i] = 0;
	}
	nBandsToGet = 128;
	sufjan.play();
	
	for (int i = 0; i < nBandsToGet; i++){
		c[i].r = ofRandom(0, 255);
		c[i].g = ofRandom(0, 255);
		c[i].b = ofRandom(0, 255);
	}
	
	for (int i = 0; i < nBandsToGet*2; i++){
		particle myParticle;
		myParticle.setInitialCondition(ofRandom(0, ofGetWidth()),ofRandom(0, ofGetHeight()),0,0);
		particles.push_back(myParticle);
	}
}


//--------------------------------------------------------------
void testApp::update(){
	ofBackground(0,0,0);

	// update the sound playing system:
	ofSoundUpdate();	
	sufjan.setVolume(2);

	// (5) grab the fft, and put in into a "smoothed" array,
	//		by taking maximums, as peaks and then smoothing downward
	float * val = ofSoundGetSpectrum(nBandsToGet);		// request 128 values for fft
	for (int i = 0;i < nBandsToGet; i++){
		
		// let the smoothed value sink to zero:
		fftSmoothed[i] *= 0.96f;
		
		// take the max, either the smoothed or the incoming:
		if (fftSmoothed[i] < val[i]) fftSmoothed[i] = val[i];
		
	}
	
	for (int i = 0; i < particles.size(); i++){
		particles[i].resetForce();
	}
	
	int count = 0;
	for (int i = 0; i < particles.size(); i++){
		
		particles[i].addAttractionForce(ofGetWidth()/2, ofGetHeight()/2, 600, 0.001f);
		
		for (int j = 0; j < i; j++){
			count++;
			particles[i].addAttractionForce(particles[j], 100, 0.1);
			particles[i].addRepulsionForce(particles[j], 30, 1.0);
		}
	}
	
//	printf("doing %i calculations \n", count);
	
	for (int i = 0; i < particles.size(); i++){
		particles[i].addDampingForce();
		particles[i].update();
		
		if (particles[i].pos.x < 0 || particles[i].pos.x > ofGetWidth() || particles[i].pos.y < 0 || particles[i].pos.y > ofGetHeight()) {
			particles.erase(particles.begin());
			particle myParticle;
			myParticle.setInitialCondition(ofRandom(0, ofGetWidth()),ofRandom(0, ofGetHeight()),0,0);
			particles.push_back(myParticle);
		}
 	}

}

//--------------------------------------------------------------
void testApp::draw(){
	ofEnableAlphaBlending();
	
	// draw the fft resutls:
	ofSetColor(255,255,255,100);
	
	ofPoint outer[nBandsToGet];
	
	ofPushMatrix();
	ofTranslate(ofGetWidth()/2, ofGetHeight()/2, 0);
		for(int i=0;i<nBandsToGet;i++){
			float cycle = ofMap(i, 0, nBandsToGet/2, 0, TWO_PI);
		
			outer[i].x = (50 * sin(cycle)) * (fftSmoothed[i]*200);
			outer[i].y = (50 * cos(cycle)) * (fftSmoothed[i]*200);
			
			ofSetColor(c[i].r, c[i].g, c[i].b,50);
			ofSetLineWidth(12);
			ofLine(0, 0, outer[i].x, outer[i].y);
			
		}
	ofPopMatrix();
	

	
	for (int i = 0; i < particles.size(); i++){
		particles[i].draw();
	}
	
	ofDisableAlphaBlending();
}


//--------------------------------------------------------------
void testApp::keyPressed  (int key){ 
}

//--------------------------------------------------------------
void testApp::keyReleased(int key){ 
	
}

//--------------------------------------------------------------
void testApp::mouseMoved(int x, int y ){
	
}

//--------------------------------------------------------------
void testApp::mouseDragged(int x, int y, int button){
	
}
 
//--------------------------------------------------------------
void testApp::mousePressed(int x, int y, int button){
//	prevx = x;
//	prevy = y;
}

//--------------------------------------------------------------
void testApp::mouseReleased(int x, int y, int button){
	

}

//--------------------------------------------------------------
void testApp::windowResized(int w, int h){

}

